package bookExamples.ch26Graphics.draw2d;

import gui.ClosableJFrame;
import gui.In;
import gui.run.RunCheckBoxMenuItem;
import gui.run.RunMenu;
import gui.run.RunMenuBar;
import gui.run.RunMenuItem;
import j2d.ImageUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

// randeep  - ImageShape
// ed - SquareShape
// gino - Cone2d
// dave - PentagonShape
// keith - RightTriangleShape
// damian - HexagonShape
// ian - CylinderShape
// enrique - EquilateralShape

public class DrawComponent extends JComponent
        implements MouseListener, MouseMotionListener {
    Font font = new Font("Times", Font.PLAIN, 12);

    private Shapes shapeList = new Shapes(0, 0, 0, 0);
    private int x1, y1;
    private Point p2 = new Point();
    private Point lastPoint = new Point();
    private DJShape lastSelectedDJShape = new Line2d(0, 0, 0, 0);
    private boolean addAShapeMode = true;

  
    public DrawComponent() {
        addMouseMotionListener(this);
        addMouseListener(this);
    }


    private void ray() {
        shapeList.addLineToLastCircle();
        repaint();
    }


    private void erase() {
        shapeList = new Shapes(0, 0, 0, 0);
        repaint();
    }


    private RunMenuBar getRunMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        mb.add(getColorMenu());
        mb.add(getDrawMenu());
        return mb;
    }

    private void outputToPrinter() {
        ImageUtils.print(this);
    }

    private RunMenu getDrawMenu() {
        RunMenu drawMenu = new RunMenu("[Draw");
        drawMenu.add(new RunCheckBoxMenuItem("add shape", true) {
            public void run() {
                addAShapeMode = getState();
                if (!addAShapeMode)
                    setText("select a shape");
                else
                    setText("add shape");
            }
        });
        drawMenu.add(new RunMenuItem("sine wave{alt control N}") {
            public void run() {
                final SineFcn f = new SineFcn(100, 100, 100);
                final FunctionShape lastSelectedJShape
                        = new FunctionShape(x1, y1, p2.x, p2.y,
                                f, 0.01);
                mediator(f);
                setLastSelectedShape(lastSelectedJShape);
            }
        });
        drawMenu.add(new RunMenuItem("s[piral{alt control P}") {
            public void run() {
                setLastSelectedShape(new Spiral(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("le[ns{alt control L}") {
            public void run() {
                setLastSelectedShape(new Lens2d(x1, y1, p2.x, p2.y));
            }
        });

        drawMenu.add(new RunMenuItem("setRotationTo90") {
            public void run() {
                AffineTransform at = getLastSelectedShape()
                        .getAffineTransform();
                at.setToRotation(90 * Math.PI / 180, x1, y1);
                DrawComponent.this.repaint();
            }
        });
        drawMenu.add(new RunMenuItem("[erase{control E}") {
            public void run() {
                erase();
            }
        });
        drawMenu.add(new RunMenuItem("[line{control L}") {
            public void run() {
                setLastSelectedShape(new Line2d(0, 0, 0, 0));
            }
        });
        //
        drawMenu.add(new RunMenuItem("R[hombus2d{control H}") {
            public void run() {
                setLastSelectedShape(new Rhombus2d(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("Pentagon[5{control 5}") {
            public void run() {
                setLastSelectedShape(new Pentagon(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("Sq[uareShape2{control U}") {
            public void run() {
                setLastSelectedShape(new SquareShape2(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("RtTrian[gle2d{alt T}") {
            public void run() {
                setLastSelectedShape(new RtTriangle2d(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("C[ylinder{control Y}") {
            public void run() {
                setLastSelectedShape(new Cylinder(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("C[one{control K}") {
            public void run() {
                setLastSelectedShape(new Cone2d(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("[ImageShape{control I}") {
            public void run() {
                setLastSelectedShape(new ImageDJShape(x1, y1, p2.x, p2.y));
            }
        });
        drawMenu.add(new RunMenuItem("[triangle{control T}") {
            public void run() {
                setLastSelectedShape(new Triangle2d(0, 0, 0, 0));
            }
        });
        drawMenu.add(new RunMenuItem("[circle{control C}") {
            public void run() {
                setLastSelectedShape(new Circle2d(0, 0, 0, 0));
            }
        });
        drawMenu.add(new RunMenuItem("o[val{control O}") {
            public void run() {
                setLastSelectedShape(new Oval2d(0, 0, 0, 0));
            }
        });
        drawMenu.add(new RunMenuItem("[rect{control 6}") {
            public void run() {
                setLastSelectedShape(new Rect2d(0, 0, 0, 0));
            }
        });
        drawMenu.add(new RunMenuItem("[FillRect2d{control F}") {
            public void run() {
                setLastSelectedShape(new FillRect2d(0, 0, 0, 0));
            }
        });
        drawMenu.add(new RunMenuItem("[string{control s}") {
            public void run() {
                setLastSelectedShape(new String2d(p2.x, p2.y, "String me!"));
            }
        });
        drawMenu.add(new RunMenuItem("r[ay{control R}") {
            public void run() {
                ray();
            }
        });
        return drawMenu;
    }

    private void mediator(final PropertyEditorFcn1 pef) {
        PropertyChangeListener pcl =
                new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent pce) {
                        PropertyEditorFcn1 f =
                                (PropertyEditorFcn1) pce.getSource();

                        final FunctionShape lastSelectedJShape
                                = new FunctionShape(x1, y1, p2.x, p2.y,
                                        f, 0.01);
                        setLastSelectedShape(lastSelectedJShape);
                        shapeList.setLastShape(lastSelectedJShape);
                        DrawComponent.this.repaint();
                    }
                };
        pef.addPropertyChangeListener(pcl);
        displayPropertyEditor(pef);
    }

    private void displayPropertyEditor(PropertyEditorFcn1 f) {
        Component comp = f.getCustomEditor();
        JDialog jd = new JDialog();
        jd.setModal(false);
        Container c = jd.getContentPane();
        c.add(comp);
        jd.setSize(200, 100);
        jd.pack();
        jd.setVisible(true);
    }

    private RunMenu getColorMenu() {
        RunMenu colorMenu = new RunMenu("[Color");
        colorMenu.add(new RunMenuItem("set [background color{alt control c}") {
            public void run() {
                setBackgroundColor();
            }
        });
        colorMenu.add(new RunMenuItem("set [foreground color{alt control f}") {
            public void run() {
                setForegroundColor();
            }
        });

        return colorMenu;
    }

    private void setForegroundColor() {
        getParent().setForeground(In.getColor());
    }

    private void setBackgroundColor() {
        getParent().setBackground(In.getColor());
    }

    private RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");

        fileMenu.add(new RunMenuItem("[print{control p}") {
            public void run() {
                outputToPrinter();
            }
        });

        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            }
        });


        return fileMenu;
    }


    public void mouseDragged(MouseEvent e) {
        e.consume();
        lastPoint.x = p2.x;
        lastPoint.y = p2.y;
        p2.x = e.getX();
        p2.y = e.getY();
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        e.consume();
        x1 = e.getX();
        y1 = e.getY();
        int x21 = -1;
        p2.x = x21;
        repaint();
    }

    public void mouseReleased(MouseEvent e) {
        if (!addAShapeMode) return;
        e.consume();
        int x2 = e.getX();
        int y2 = e.getY();
        shapeList.addShape(getLastSelectedShape().getInstance(x1, y1, x2, y2));
        int xl1 = -1;
        lastPoint.x = xl1;
        int x21 = -1;
        p2.x = x21;
        repaint();
    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {
        repaint();
    }

    public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
            Point p = e.getPoint();
            DJShape s = shapeList.getClosestShape(p);
            s.setSelected(true);
            repaint();
        }
    }

    public void drawRubberBand(Graphics g) {
        if (!addAShapeMode) return;
        if (p2.x == -1) return;
        g.setXORMode(getBackground());
        getLastSelectedShape().getInstance(x1, y1, p2.x, p2.y).draw(g);
    }

    public void paintComponent(Graphics g) {

        super.paintComponent(g);
        g.setFont(font);
        shapeList.draw(g);
        drawRubberBand(g);

    }


    public static void main(String args[]) {
        DrawComponent drawComponent = new DrawComponent();
        ClosableJFrame cf = new ClosableJFrame("DocJavaDraw");
        cf.setJMenuBar(drawComponent.getRunMenuBar());
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(drawComponent, BorderLayout.CENTER);
        cf.setSize(300, 300);
        cf.setVisible(true);
    }


    public DJShape getLastSelectedShape() {
        return lastSelectedDJShape;
    }

    public void setLastSelectedShape(DJShape lastSelectedDJShape) {
        this.lastSelectedDJShape = lastSelectedDJShape;

        repaint();
    }
}













